@@ -33,8 +33,11 @@ __all__ = [
 from syspassclient.constants import Constants
 from syspassclient.constants import dd
 from syspassclient.object import Object
+from syspassclient.api import SyspassApi
 from syspassclient.config import Config
-config = Config()
 from syspassclient.check_type import CheckType
 from syspassclient.utils import Utils
 from syspassclient.syspassclient import SyspassClient
+
+# require for digdeo-syspass-ansible-lookup
+config = Config()
