@@ -43,13 +43,20 @@ except ImportError:
     from yaml import Loader, Dumper
 
 
-class Singleton(type):
-    instance = None
+import threading
 
-    def __call__(cls, *args, **kw):
-        if not cls.instance:
-            cls.instance = super(Singleton, cls).__call__(*args, **kw)
-        return cls.instance
+lock = threading.Lock()
+
+
+class Singleton(type):
+    _instances = {}
+
+    def __call__(cls, *args, **kwargs):
+        if cls not in cls._instances:
+            with lock:
+                if cls not in cls._instances:
+                    cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
+        return cls._instances[cls]
 
 
 class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singleton):
@@ -64,16 +71,31 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         self.__r_id = None
 
         # Initialize
-        self.api_url = syspassclient.config.get_data()['api_url']
-        self.authToken = syspassclient.config.get_data()['authToken']
-        self.tokenPass = syspassclient.config.get_data()['tokenPass']
-        self.debug = syspassclient.config.debug
-        self.debug_level = syspassclient.config.debug_level
-        self.verbose = syspassclient.config.verbose
-        self.verbose_level = syspassclient.config.verbose_level
+        self.__config = None
+        self.config = syspassclient.config
+        self.api_url = self.config.api_url
+        # self.authToken = self.config.authToken
+        # self.tokenPass = self.config.tokenPass
+        self.debug = self.config.debug
+        self.debug_level = self.config.debug_level
+        self.verbose = self.config.verbose
+        self.verbose_level = self.config.verbose_level
 
         self.r_id = 1
 
+    @property
+    def config(self):
+        return self.__config
+
+    @config.setter
+    def config(self, value=None):
+        if value is None and self.config is None:
+            return
+        if value is not None and not isinstance(value, syspassclient.Config):
+            raise TypeError('"config" value must be a a syspassclient.Config instance')
+        if self.config != value:
+            self.__config = value
+
     @property
     def api_url(self):
         """
@@ -105,66 +127,67 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         if self.__api_url != api_url:
             self.__api_url = api_url
 
-    @property
-    def authToken(self):
-        """
-        Whether the application will paint directly on the widget.
+    # @property
+    # def authToken(self):
+    #     """
+    #     Whether the application will paint directly on the widget.
+    #
+    #     :return: True or False
+    #     :rtype: bool
+    #     """
+    #     return self.config.authToken
 
-        :return: True or False
-        :rtype: bool
-        """
-        return self.__authToken
+    # @authToken.setter
+    # def authToken(self, authToken=None):
+    #     """
+    #     Set ``api_key`` property
+    #
+    #     :param authToken: A SysPass API Key
+    #     :type authToken: str or None
+    #     :raise TypeError: if ``api_key`` is not a str of None
+    #     """
+    #     if authToken is None or authToken == "":
+    #         if self.__authToken != authToken:
+    #             self.__authToken = None
+    #         return
+    #
+    #     self.is_str_or_raise(authToken)
+    #     self.is_ascii_or_raise(authToken)
+    #
+    #     if self.__authToken != authToken:
+    #         self.__authToken = authToken
 
-    @authToken.setter
-    def authToken(self, authToken=None):
-        """
-        Set ``api_key`` property
-
-        :param authToken: A SysPass API Key
-        :type authToken: str or None
-        :raise TypeError: if ``api_key`` is not a str of None
-        """
-        if authToken is None or authToken == "":
-            if self.__authToken != authToken:
-                self.__authToken = None
-            return
-
-        self.is_str_or_raise(authToken)
-        self.is_ascii_or_raise(authToken)
-
-        if self.__authToken != authToken:
-            self.__authToken = authToken
-
-    @property
-    def tokenPass(self):
-        """
-        Whether the application will paint directly on the widget.
-
-        :return: True or False
-        :rtype: bool
-        """
-        return self.__tokenPass
-
-    @tokenPass.setter
-    def tokenPass(self, tokenPass=None):
-        """
-        Set ``api_acc_tokpwd`` property
-
-        :param tokenPass: A SysPass API Key
-        :type tokenPass: str or None
-        :raise TypeError: if ``api_acc_tokpwd`` is not a str of None
-        """
-        if tokenPass is None or tokenPass == "":
-            if self.__tokenPass != tokenPass:
-                self.__tokenPass = None
-            return
-
-        if tokenPass is not None:
-            self.is_str_or_raise(tokenPass)
-            self.is_ascii_or_raise(tokenPass)
-
-        if self.__tokenPass != tokenPass:
-            self.__tokenPass = tokenPass
+    # @property
+    # def tokenPass(self):
+    #     """
+    #     Whether the application will paint directly on the widget.
+    #
+    #     :return: True or False
+    #     :rtype: bool
+    #     """
+    #     # if self.config.rw:
+    #     return self.config.tokenPass
+    #
+    # @tokenPass.setter
+    # def tokenPass(self, tokenPass=None):
+    #     """
+    #     Set ``api_acc_tokpwd`` property
+    #
+    #     :param tokenPass: A SysPass API Key
+    #     :type tokenPass: str or None
+    #     :raise TypeError: if ``api_acc_tokpwd`` is not a str of None
+    #     """
+    #     if tokenPass is None or tokenPass == "":
+    #         if self.__tokenPass != tokenPass:
+    #             self.__tokenPass = None
+    #         return
+    #
+    #     if tokenPass is not None:
+    #         self.is_str_or_raise(tokenPass)
+    #         self.is_ascii_or_raise(tokenPass)
+    #
+    #     if self.__tokenPass != tokenPass:
+    #         self.__tokenPass = tokenPass
 
     def make_post_request(self, data=None, verify=False):
         """
@@ -337,7 +360,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         :rtype: str in case of success or None if nothing found
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         # Exit as soon of possible
         self.look_for_args(
@@ -400,7 +423,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         :type account_id: int
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         # Exit as soon of possible
         self.look_for_args(method=method, authToken=authToken, account_id=account_id)
@@ -441,10 +464,10 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         :type account_id: int
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         if tokenPass is None:
-            tokenPass = self.tokenPass
+            tokenPass = self.config.tokenPass
 
         self.look_for_args(method=method, authToken=authToken, tokenPass=tokenPass, account_id=account_id)
 
@@ -488,10 +511,10 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         :type details: int or None
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         if tokenPass is None:
-            tokenPass = self.tokenPass
+            tokenPass = self.config.tokenPass
 
         self.look_for_args(
             method=method,
@@ -601,12 +624,12 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         :type expireDate: int
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
         if authToken is None:
             raise TypeError("'authToken' must be set")
 
         if tokenPass is None:
-            tokenPass = self.tokenPass
+            tokenPass = self.config.tokenPass
 
         if tokenPass is None:
             raise TypeError("'tokenPass' must be set")
@@ -670,7 +693,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         count is the number of results.
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         self.look_for_args(method=method, authToken=authToken, text=text, count=count)
 
@@ -715,7 +738,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         :type description: str
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         self.look_for_args(method=method, authToken=authToken, name=name, description=description)
 
@@ -747,7 +770,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         cid 	    int 	yes 	    Category’s Id
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         self.look_for_args(method=method, authToken=authToken, cid=cid)
 
@@ -761,7 +784,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
 
         self.look_for_error(data, req)
 
-        print(req)
+        # print(req)
         if req["result"]["resultCode"] == 0:
             return req["result"]
         # else:
@@ -778,7 +801,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         count 	    int 	no 	        Number of results to display
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         self.look_for_args(method=method, authToken=authToken, text=text, count=count)
 
@@ -818,7 +841,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         :type Global: int
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         if Global is not None:
             Global = int(bool(Global))
@@ -858,7 +881,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         :type cid: int
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         self.look_for_args(method=method, authToken=authToken, cid=cid)
 
@@ -895,7 +918,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         :type text: str
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         self.look_for_args(method=method, authToken=authToken, text=text)
 
@@ -939,10 +962,10 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         :type tagid: int
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         if tokenPass is None:
-            tokenPass = self.tokenPass
+            tokenPass = self.config.tokenPass
 
         self.look_for_args(method=method, authToken=authToken, tokenPass=tokenPass, tagid=tagid)
 
@@ -979,7 +1002,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         :type name: str
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         self.look_for_args(method=method, authToken=authToken, name=name)
 
@@ -998,6 +1021,16 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
 
         return self.tag_search(text=name)
 
+
+        # if self.config.rw:
+        #     return self.tag_search(text=name)
+        # else:
+        #     to_return = self.tag_search(text=name)
+        #     if to_return is None:
+        #         raise ValueError('impossible to creat the tag')
+        #     return to_return
+
+
     def tag_delete(self, method='tag/delete', authToken=None, tagid=None):
         """
         tag/delete
@@ -1016,7 +1049,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         :type tagid: int
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         self.look_for_args(method=method, authToken=authToken, tagid=tagid)
 
@@ -1063,7 +1096,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         :type usersId: list
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         self.look_for_args(method=method, authToken=authToken, name=name, description=description, usersId=usersId)
         data = self.generate_json(
@@ -1092,7 +1125,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         """
 
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         self.look_for_args(method=method, authToken=authToken, text=text, count=count)
         data = self.generate_json(method=method, authToken=authToken, text=text, count=count)
@@ -1128,7 +1161,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         :type ugid: int
         """
         if authToken is None:
-            authToken = self.authToken
+            authToken = self.config.authToken
 
         self.look_for_args(method=method, authToken=authToken, ugid=ugid)
         data = self.generate_json(method=method, authToken=authToken, ugid=ugid)
